@@ -6,9 +6,10 @@ use Mojo::ByteStream;
 use Mojo::Collection;
 use Mojo::Exception;
 use Mojo::IOLoop;
+use Mojo::Promise;
 use Mojo::Util qw(dumper hmac_sha1_sum steady_time);
 use Time::HiRes qw(gettimeofday tv_interval);
-use Scalar::Util 'blessed';
+use Scalar::Util qw(blessed weaken);
 
 sub register {
   my ($self, $app) = @_;
@@ -38,9 +39,13 @@ sub register {
     for qw(csrf_token current_route flash inactivity_timeout is_fresh),
     qw(redirect_to respond_to url_with validation);
 
-  $app->helper(dumper => sub { shift; dumper @_ });
+  $app->helper(dumper  => sub { shift; dumper @_ });
   $app->helper(include => sub { shift->render_to_string(@_) });
 
+  $app->helper('proxy.get_p'  => sub { _proxy_method_p('GET',  @_) });
+  $app->helper('proxy.post_p' => sub { _proxy_method_p('POST', @_) });
+  $app->helper('proxy.start_p' => \&_proxy_start_p);
+
   $app->helper("reply.$_" => $self->can("_$_")) for qw(asset file static);
 
   $app->helper('reply.exception' => sub { _development('exception', @_) });
@@ -68,9 +73,9 @@ sub _content {
 
   my $hash = $c->stash->{'mojo.content'} ||= {};
   if (defined $content) {
-    if ($append) { $hash->{$name} .= _block($content) }
-    if ($replace) { $hash->{$name} = _block($content) }
-    else          { $hash->{$name} //= _block($content) }
+    if   ($append)  { $hash->{$name} .= _block($content) }
+    if   ($replace) { $hash->{$name} = _block($content) }
+    else            { $hash->{$name} //= _block($content) }
   }
 
   return Mojo::ByteStream->new($hash->{$name} // '');
@@ -161,6 +166,56 @@ sub _is_fresh {
   return $c->app->static->is_fresh($c, \%options);
 }
 
+sub _proxy_method_p {
+  my ($method, $c) = (shift, shift);
+  return _proxy_start_p($c, $c->ua->build_tx($method, @_));
+}
+
+sub _proxy_start_p {
+  my ($c, $source_tx) = @_;
+  my $tx = $c->render_later->tx;
+
+  my $promise = Mojo::Promise->new;
+  $source_tx->res->content->auto_upgrade(0)->auto_decompress(0)->once(
+    body => sub {
+      my $source_content = shift;
+
+      my $source_res = $source_tx->res;
+      my $res        = $tx->res;
+      my $content    = $res->content;
+      $res->code($source_res->code)->message($source_res->message);
+      my $headers = $source_res->headers->clone->dehop;
+      $content->headers($headers);
+      $promise->resolve;
+
+      my $source_stream = Mojo::IOLoop->stream($source_tx->connection);
+      return unless my $stream = Mojo::IOLoop->stream($tx->connection);
+
+      my $write = $source_content->is_chunked ? 'write_chunk' : 'write';
+      $source_content->unsubscribe('read')->on(
+        read => sub {
+          $content->$write(pop) and $tx->resume;
+
+          # Throttle transparently when backpressure rises
+          return if $stream->can_write;
+          $source_stream->stop;
+          $stream->once(drain => sub { $source_stream->start });
+        }
+      );
+
+      # Unknown length (fall back to connection close)
+      $source_res->once(finish => sub { $content->$write('') and $tx->resume })
+        unless length($headers->content_length // '');
+    }
+  );
+  weaken $source_tx;
+  $source_tx->once(finish => sub { $promise->reject(_tx_error(@_)) });
+
+  $c->ua->start_p($source_tx);
+
+  return $promise;
+}
+
 sub _redirect_to {
   my $c = shift;
 
@@ -220,6 +275,8 @@ sub _timing_server_timing {
   $c->res->headers->append('Server-Timing' => $value);
 }
 
+sub _tx_error { (shift->error || {})->{message} // 'Unknown error' }
+
 sub _url_with {
   my $c = shift;
   return $c->url_for(@_)->query($c->req->url->query->clone);
@@ -466,6 +523,63 @@ L</"stash">.
 
 Alias for L<Mojolicious::Controller/"param">.
 
+=head2 proxy->get_p
+
+  my $promise = $c->proxy->get_p('http://example.com' => {Accept => '*/*'});
+
+Perform non-blocking C<GET> request and forward response as efficiently as
+possible, takes the same arguments as L<Mojo::UserAgent/"get"> and returns a
+L<Mojo::Promise> object. Note that this helper is EXPERIMENTAL and might change
+without warning!
+
+  # Forward with exception handling
+  $c->proxy->get_p('http://mojolicious.org')->catch(sub {
+    my $err = shift;
+    $c->log->debug("Proxy error: $err");
+    $c->render(text => 'Something went wrong!', status => 400);
+  });
+
+=head2 proxy->post_p
+
+  my $promise = $c->proxy->post_p('http://example.com' => {Accept => '*/*'});
+
+Perform non-blocking C<POST> request and forward response as efficiently as
+possible, takes the same arguments as L<Mojo::UserAgent/"post"> and returns a
+L<Mojo::Promise> object. Note that this helper is EXPERIMENTAL and might change
+without warning!
+
+  # Forward with exception handling
+  $c->proxy->post_p('example.com' => form => {test => 'pass'})->catch(sub {
+    my $err = shift;
+    $c->log->debug("Proxy error: $err");
+    $c->render(text => 'Something went wrong!', status => 400);
+  });
+
+=head2 proxy->start_p
+
+  my $promise = $c->proxy->start_p(Mojo::Transaction::HTTP->new);
+
+Perform non-blocking request for a custom L<Mojo::Transaction::HTTP> object and
+forward response as efficiently as possible, returns a L<Mojo::Promise> object.
+Note that this helper is EXPERIMENTAL and might change without warning!
+
+  # Forward with exception handling
+  my $tx = $c->ua->build_tx(GET => 'http://mojolicious.org');
+  $c->proxy->start_p($tx)->catch(sub {
+    my $err = shift;
+    $c->log->debug("Proxy error: $err");
+    $c->render(text => 'Something went wrong!', status => 400);
+  });
+
+  # Forward with custom request and response headers
+  my $headers = $c->req->headers->clone->dehop;
+  $headers->header('X-Proxy' => 'Mojo');
+  my $tx = $c->ua->build_tx(GET => 'http://example.com' => $headers->to_hash);
+  $c->proxy->start_p($tx);
+  $tx->res->content->once(body => sub {
+    $c->res->headers->header('X-Proxy' => 'Mojo');
+  });
+
 =head2 redirect_to
 
   $c = $c->redirect_to('named', foo => 'bar');
